function ffe(fractal_ref)
{
    //alert("ffe::ctor()");
    this.m_fractal = fractal_ref;
    
    
    // Public key
    //___________________________________________________
    this.public_key = function(xmin, xmax, ymin, ymax)
    {
        //alert("ffe::public_key::ctor()");
        
        this.m_axes = new fractal_ref.axes(xmin, xmax, ymin, ymax);
    };
    
    this.public_key.prototype.copy = function(rhs)
    {
        //alert("ffe::public_key::copy()");
        this.m_axes.copy(rhs.m_axes);
    };
    
    
    // Private key
    //___________________________________________________
    this.private_key = function(
        xmin, xmax, ymin, ymax,
        jx, jy, 
        dimpad, etseed_in, etseed_out, 
        imax, 
        encrypt, decrypt)
    {
        //alert("ffe::private_key::ctor()");
        
        this.m_axes = new fractal_ref.axes(xmin, xmax, ymin, ymax);
        this.m_etseed_in = etseed_in;
        this.m_etseed_out = etseed_out;
        this.m_imax = imax;
        this.m_dimpad = dimpad;
        this.m_j = new fractal_ref.point(jx, jy);
        this.m_encrypt = encrypt;
        this.m_decrypt = decrypt;
    };
    
    this.private_key.prototype.encrypt_raw_byte = function(raw_byte, z, j, i, imax)
    {
        //alert("ffe::private_key::encrypt_raw_byte()");
        
        var cipher_byte = this.m_encrypt(raw_byte, z, j, i, imax);
        
        return cipher_byte;
    };
    
    this.private_key.prototype.decrypt_raw_byte = function(raw_byte, z, j, i, imax)
    {
        //alert("ffe::private_key::encrypt_raw_byte()");
        
        var cipher_byte = this.m_decrypt(raw_byte, z, j, i, imax);
        
        return cipher_byte;
    };
    
    
    // Cipher Result
    this.cipher_result = function(public_key, cipher_bytes)
    {
        this.m_public_key = public_key;
        this.m_cipher_bytes = cipher_bytes;
    };
    
    this.cipher_result.prototype.get_key = function()
    {
        //alert("ffe::private_key::get_key()");
        
        return this.m_public_key;
    };
    
    this.cipher_result.prototype.get_bytes = function()
    {
        //alert("ffe::private_key::get_bytes()");
        
        return this.m_cipher_bytes;
    };
    
    
    
    this.m_public_key = null;
    this.m_private_key = null;
    this.m_raw_bytes = null;
    this.m_cipher_bytes = null;
    this.m_raw_bytes_size = 0;
    
    this.m_cipher_result = null;
    
    this.m_vport = new fractal_ref.viewport(0, 0);
    this.m_z = new fractal_ref.point(0.0, 0.0);
    this.m_c = new fractal_ref.point(0.0, 0.0);
    this.m_ir = new fractal_ref.iter_result();
}


ffe.prototype.set_state = function(raw_bytes, cipher_result, private_key)
{
    this.m_public_key = cipher_result.get_key();
    this.m_private_key = private_key;
    this.m_raw_bytes = raw_bytes;
    this.m_raw_bytes_size = raw_bytes.length;
    this.m_cipher_bytes = "";
    this.m_cipher_result = cipher_result;
};


ffe.prototype.clear_state = function()
{
    this.m_public_key = null;
    this.m_private_key = null;
    this.m_raw_bytes = null;
    this.m_raw_bytes_size = 0;
    this.m_cipher_bytes = null;

    var cipher_result = this.m_cipher_result;
    this.m_cipher_result = null;
        
    return cipher_result;
};


ffe.prototype.encrypt_raw_byte = function(z, j, i, imax)
{
    if (i >= this.m_raw_bytes_size) return false;
    
    var raw_byte = this.m_raw_bytes.charCodeAt(i);
    var cipher_byte = this.m_private_key.encrypt_raw_byte(raw_byte, z, j, i, imax);
    
    this.m_cipher_result.m_cipher_bytes += String.fromCharCode(cipher_byte);
    
    //alert("ffe::encrypt_raw_byte[" + i + "]:cipher_byte = " + cipher_byte);
    
    return true;
};


ffe.prototype.decrypt_raw_byte = function(z, j, i, imax)
{
    if (i >= this.m_raw_bytes_size) return false;
    
    var raw_byte = this.m_raw_bytes.charCodeAt(i);
    var cipher_byte = this.m_private_key.decrypt_raw_byte(raw_byte, z, j, i, imax);
    
    this.m_cipher_result.m_cipher_bytes += String.fromCharCode(cipher_byte);
    
    //alert("ffe::decrypt_raw_byte[" + i + "]:cipher_byte = " + cipher_byte);
    
    return true;
};


ffe.prototype.generate_public_key = function(raw_bytes, private_key)
{
    if (! raw_bytes.length) return null;
    
    var imax = raw_bytes.length;
    var imod = 9999999999999999;
    var cpack0 = raw_bytes.charCodeAt(i);
    var cpack1 = raw_bytes.charCodeAt(i);
    var cpack2 = raw_bytes.charCodeAt(i);
    
    for (var i = 2; i < imax; ++i)
    {
        var raw_byte0 = raw_bytes.charCodeAt(i - 2);
        var raw_byte1 = raw_bytes.charCodeAt(i - 1);
        var raw_byte2 = raw_bytes.charCodeAt(i);
        
        cpack0 = this.m_fractal.cantor_pack(cpack0, raw_byte0) % imod;
        cpack1 = this.m_fractal.cantor_pack(raw_byte1, cpack0) % imod;
        cpack2 = this.m_fractal.cantor_pack(cpack1, raw_byte2) % imod;
    }
    
    var cpack3 = this.m_fractal.cantor_pack(cpack0, cpack1) % imod;
    cpack3 = this.m_fractal.cantor_pack(cpack3, cpack2) % imod;
    
    var axes_mod = 1.0 + Math.random() * 0.681;
    
    var pow_n = 0.3 + Math.random() * 0.5;
    
    var xmin_mutate = Math.pow(cpack0, pow_n) % axes_mod;
    var xmax_mutate = Math.pow(cpack1, pow_n) % axes_mod;
    var ymin_mutate = Math.pow(cpack2, pow_n) % axes_mod;
    var ymax_mutate = Math.pow(cpack3, pow_n) % axes_mod;
    
    xmin_mutate += 0.01;
    xmax_mutate += 0.01;
    ymin_mutate += 0.01;
    ymax_mutate += 0.01;
    
    var xmin = private_key.m_axes.m_xmin * xmin_mutate;
    var xmax = private_key.m_axes.m_xmax * xmax_mutate;
    var ymin = private_key.m_axes.m_ymin * ymin_mutate;
    var ymax = private_key.m_axes.m_ymax * ymax_mutate;
    
    var public_key = new this.public_key(xmin, xmax, ymin, ymax);
    
    
    return public_key;
};


ffe.prototype.encrypt = function(raw_bytes, private_key)
{
    //alert("ffe::encrypt()");
    
    if (! raw_bytes.length) return null;
    
    var public_key = this.generate_public_key(raw_bytes, private_key);
    var encrypted = new this.cipher_result(public_key, "");
    
    this.set_state(raw_bytes, encrypted, private_key);
    
    var dims = Math.ceil(Math.sqrt(this.m_raw_bytes_size)) + private_key.m_dimpad;
    
    this.m_vport.m_width = dims;
    this.m_vport.m_height = dims;
    
    this.m_fractal.mbrot_plane(
        this.m_vport, 
        public_key.m_axes, 
        this.m_z, 
        this.m_c, 
        private_key.m_j, 
        private_key.m_imax,
        this.encrypt_raw_byte.bind(this)
    );
    
    return this.clear_state();
};


ffe.prototype.decrypt = function(raw_bytes, private_key, public_key)
{
    //alert("ffe::decrypt()");
    
    if (! raw_bytes.length) return null;
    
    var decrypted = new this.cipher_result(public_key, "");
    
    this.set_state(raw_bytes, decrypted, private_key);
    
    var dims = Math.ceil(Math.sqrt(this.m_raw_bytes_size)) + private_key.m_dimpad;
    
    this.m_vport.m_width = dims;
    this.m_vport.m_height = dims;
    
    this.m_fractal.mbrot_plane(
        this.m_vport, 
        public_key.m_axes, 
        this.m_z, 
        this.m_c, 
        private_key.m_j, 
        private_key.m_imax,
        this.decrypt_raw_byte.bind(this)
    );
    
    return this.clear_state();
};
