
// The private key functions
//________________________________________________________
function private_key_formula_0(cipher)
{
    //alert("private_key_formula_0::ctor()");
    
    this.m_cipher = cipher;
   
    this.m_private_key = new cipher.private_key(
        -0.5, 0.5, -0.5, 0.5, 
        //0.35, 0.1, 
        -0.75, 0.1,
        1, 
        10030, 
        1003, 
        613,
        this.encrypt_raw_byte.bind(this),
        this.decrypt_raw_byte.bind(this)
    );
}

private_key_formula_0.prototype.get_key = function()
{
    //alert("private_key_formula_0::get_key()");
    return this.m_private_key;
};


private_key_formula_0.prototype.get_cipher_mutator = function(z, j, i, imax)
{
    var et = this.m_cipher.m_fractal.mbrot_iter(
        this.m_cipher.m_ir, this.m_cipher.m_z, j, imax
    );
    
    var cipher_mutator = 0.0;
    
    if (et != imax)
    {
        cipher_mutator = et * this.m_cipher.m_ir.m_min_distance * 
            (this.m_cipher.m_z.m_x + this.m_cipher.m_z.m_y);

        cipher_mutator *= 5134.0413;
    }
    
    else
    {
        cipher_mutator = et * this.m_cipher.m_ir.m_min_distance * 
            (this.m_cipher.m_z.m_x + this.m_cipher.m_z.m_y);
            
        cipher_mutator *= 178654.681;
    }
    
    //alert("this.m_cipher.m_ir.m_min_distance = " + this.m_cipher.m_ir.m_min_distance);
    //alert("this.m_cipher.m_z.m_x = " + this.m_cipher.m_z.m_x);
    //alert("this.m_cipher.m_z.m_y = " + this.m_cipher.m_z.m_y);
    
    cipher_mutator = Math.abs(Math.round(cipher_mutator)) % 256;
    
    //alert("cipher_mutator = " + cipher_mutator);
    
    return cipher_mutator;
};


private_key_formula_0.prototype.encrypt_raw_byte = function(raw_byte, z, j, i, imax)
{
    //alert("private_key_formula_0::encrypt_raw_byte()");
    
    var cipher_mutator = this.get_cipher_mutator(z, j, i, imax);
    
    var cipher_byte = (raw_byte + cipher_mutator) % 256;
    
    return cipher_byte;
};


private_key_formula_0.prototype.decrypt_raw_byte = function(raw_byte, z, j, i, imax)
{
    //alert("private_key_formula_0::decrypt_raw_byte()");
    
    var cipher_mutator = this.get_cipher_mutator(z, j, i, imax);
    
    var cipher_byte = raw_byte - cipher_mutator;
    
    if (cipher_byte < 0)
    {
        cipher_byte = raw_byte + 256;
        cipher_byte = Math.abs(cipher_mutator - cipher_byte);
    }

    return cipher_byte;
};






// Test application of the fractal cipher...
//________________________________________________________
var g_ffe_test_app = null;


function ffe_test_app()
{
    //alert("ffe_test_app::ctor()");
    
    this.m_fractal = new fractal();
    this.m_cipher = new ffe(this.m_fractal);
    this.m_public_key = new this.m_cipher.public_key(0.0, 0.0, 0.0, 0.0);
    this.m_pkformula = new private_key_formula_0(this.m_cipher);
    
    this.m_canvas = new mycanvas("g_html_canvas_fractal", "2d", 300, 300);
    this.m_fcanvas = new fractal_canvas(this.m_fractal, this.m_canvas);
}


ffe_test_app.prototype.render = function()
{
    if (! this.m_public_key) return false;
    var axes = this.m_public_key.m_axes;
    var jpt = this.m_pkformula.m_private_key.m_j;
    
    this.m_fcanvas.render(axes, jpt, this.m_pkformula.m_private_key.m_imax);
    
    return true;
};


ffe_test_app.prototype.encrypt = function()
{
    // get the private key from the ui
    this.parse_get_private_key(this.m_pkformula.get_key());
    
    var plaintext = ui_parse_string_from_element("g_html_input_plaintext");
    var encrypted = this.m_cipher.encrypt(plaintext, this.m_pkformula.get_key());
    
    if (! encrypted)
    {
        ui_set_element_to_string("g_html_input_ciphertext", "");
        return false;
    }
    
    var hexbytes = string_to_hexbytes(encrypted.get_bytes(), 0);
    this.m_public_key.copy(encrypted.get_key());
    
    // build the public key in text form
    var public_key_text = this.m_public_key.m_axes.m_xmin.toString();
    public_key_text += " " + this.m_public_key.m_axes.m_xmax.toString();
    public_key_text += "\r\n" + this.m_public_key.m_axes.m_ymin.toString();
    public_key_text += " " + this.m_public_key.m_axes.m_ymax.toString();
    
    var final_cipher_text = public_key_text + "\r\n\r\n" + hexbytes;
    
    ui_set_element_to_string("g_html_input_ciphertext", final_cipher_text);
   
    // Update the public key in the ui
    ui_set_element_to_string("g_html_input_public_key_xmin", this.m_public_key.m_axes.m_xmin.toString());
    ui_set_element_to_string("g_html_input_public_key_xmax", this.m_public_key.m_axes.m_xmax.toString());
    ui_set_element_to_string("g_html_input_public_key_ymin", this.m_public_key.m_axes.m_ymin.toString());
    ui_set_element_to_string("g_html_input_public_key_ymax", this.m_public_key.m_axes.m_ymax.toString());
    
    this.render();
    
    return true;
};


ffe_test_app.prototype.decrypt = function()
{
    // get the private key from the ui
    this.parse_get_private_key(this.m_pkformula.get_key());
    
    // obtain raw cipher text
    var raw_cipher_text = ui_parse_string_from_element("g_html_input_ciphertext");
    
    // get the public key from the ui
    var count = parse_ciphertext_into_public_key(raw_cipher_text, this.m_public_key, 4);
    if (count < 1) { alert("DECRYPT ERROR!!"); return; }
    
    // update the public key in the UI
    ui_set_element_to_string("g_html_input_public_key_xmin", this.m_public_key.m_axes.m_xmin.toString());
    ui_set_element_to_string("g_html_input_public_key_xmax", this.m_public_key.m_axes.m_xmax.toString());
    ui_set_element_to_string("g_html_input_public_key_ymin", this.m_public_key.m_axes.m_ymin.toString());
    ui_set_element_to_string("g_html_input_public_key_ymax", this.m_public_key.m_axes.m_ymax.toString());
    
    // get the ciphertext
    var ciphertext = hexbytes_to_string(raw_cipher_text, count);
    var decrypted = this.m_cipher.decrypt(ciphertext, this.m_pkformula.get_key(), this.m_public_key);
    
    if (! decrypted)
    {
        ui_set_element_to_string("g_html_input_plaintext", "");
        return false;
    }
    
    ui_set_element_to_string("g_html_input_plaintext", decrypted.get_bytes());
};




ffe_test_app.prototype.parse_get_private_key = function(pkey)
{
    // Julia point
    pkey.m_j.m_x = ui_parse_float_from_element("g_html_input_private_key_jx", 0.0);
    pkey.m_j.m_y = ui_parse_float_from_element("g_html_input_private_key_jy", 0.0);
    
    // Axes
    pkey.m_axes.m_xmin = ui_parse_float_from_element("g_html_input_private_key_xmin", 0.0);
    pkey.m_axes.m_xmax = ui_parse_float_from_element("g_html_input_private_key_xmax", 0.0);
    pkey.m_axes.m_ymin = ui_parse_float_from_element("g_html_input_private_key_ymin", 0.0);
    pkey.m_axes.m_ymax = ui_parse_float_from_element("g_html_input_private_key_ymax", 0.0);
    
    /*
     * this.m_axes = new fractal_ref.axes(xmin, xmax, ymin, ymax);
        this.m_etseed_in = etseed_in;
        this.m_etseed_out = etseed_out;
        this.m_imax = imax;
        this.m_dimpad = dimpad;
        this.m_j = new fractal_ref.point(jx, jy);
        this.m_encrypt = encrypt;
        this.m_decrypt = decrypt;
     */
};


ffe_test_app.prototype.parse_get_public_key = function(pkey)
{
    // Axes
    pkey.m_axes.m_xmin = ui_parse_float_from_element("g_html_input_public_key_xmin", 0.0);
    pkey.m_axes.m_xmax = ui_parse_float_from_element("g_html_input_public_key_xmax", 0.0);
    pkey.m_axes.m_ymin = ui_parse_float_from_element("g_html_input_public_key_ymin", 0.0);
    pkey.m_axes.m_ymax = ui_parse_float_from_element("g_html_input_public_key_ymax", 0.0);
};

/*
 * // get the public key from the ui
    this.m_public_key.m_axes.m_xmin = ui_parse_float_from_element("g_html_input_public_key_xmin", 0.0);
    this.m_public_key.m_axes.m_xmax = ui_parse_float_from_element("g_html_input_public_key_xmax", 0.0);
    this.m_public_key.m_axes.m_ymin = ui_parse_float_from_element("g_html_input_public_key_ymin", 0.0);
    this.m_public_key.m_axes.m_ymax = ui_parse_float_from_element("g_html_input_public_key_ymax", 0.0);
 */




// HTML UI Event Handlers
//________________________________________________________
function ffe_test_app_ui_encrypt_on_click()
{
    if (! g_ffe_test_app) return;
    //alert("ffe_test_app_ui_encrypt_on_click");
    g_ffe_test_app.encrypt();
}

function ffe_test_app_ui_decrypt_on_click()
{
    if (! g_ffe_test_app) return;
    //alert("ffe_test_app_ui_decrypt_on_click");
    g_ffe_test_app.decrypt();
}




// Application init
//________________________________________________________
function ffe_test_app_main()
{
    if (g_ffe_test_app) return;
    //alert("ffe_test_app_main");
    g_ffe_test_app = new ffe_test_app();
    
    g_ffe_test_app.encrypt();
}
