function fractal()
{
    //alert("fractal::ctor()");
    
    
    
    
    // Plane Axes
    //___________________________________________________
    this.axes = function(xmin, xmax, ymin, ymax)
    {
        //alert("fractal::axes::ctor()");
        
        this.m_xmin = xmin;
        this.m_xmax = xmax;
        this.m_ymin = ymin;
        this.m_ymax = ymax;
    };
    
    this.axes.prototype.copy = function(rhs)
    {
        //alert("fractal::axes::copy()");
        this.m_xmin = rhs.m_xmin;
        this.m_xmax = rhs.m_xmax;
        this.m_ymin = rhs.m_ymin;
        this.m_ymax = rhs.m_ymax;
    };
    
    this.axes.prototype.get_width = function()
    {
        //alert("fractal::axes::get_width()");
        return this.m_xmax - this.m_xmin;
    };
    
    this.axes.prototype.get_height = function()
    {
        //alert("fractal::axes::get_height()");
        return this.m_ymax - this.m_ymin;
    };
    
    
    
    
    // Point
    //___________________________________________________
    this.point = function(x, y)
    {
        //alert("fractal::point::ctor()");
        
        this.m_x = x;
        this.m_y = y;
    };
    
    this.point.prototype.copy = function(rhs)
    {
        //alert("fractal::point::copy()");
        this.m_x = rhs.m_x;
        this.m_y = rhs.m_y;
    };
    
    this.point.prototype.distance = function()
    {
        //alert("fractal::point::distance()");
        return Math.sqrt(this.m_x * this.m_x + this.m_y * this.m_y);
    };
    
    
    
    // Viewport
    //___________________________________________________
    this.viewport = function(width, height)
    {
        //alert("fractal::viewport::ctor()");
        
        this.m_width = width;
        this.m_height = height;
        this.m_xstep = 0;
        this.m_ystep = 0;
    };
    
    this.viewport.prototype.set_step = function(caxes)
    {
        //alert("fractal::viewport::set_step()");
        var sx = (this.m_width > 1) ? (this.m_width - 1) : (1);
        var sy = (this.m_height > 1) ? (this.m_height - 1) : (1);
        this.m_xstep = caxes.get_width() / sx;
        this.m_ystep = caxes.get_height() / sy;
    };
    
    this.viewport.prototype.get_point = function(caxes, pt, x, y)
    {
        //alert("fractal::viewport::get_point()");
        pt.m_x = caxes.m_xmin + this.m_xstep * x;
        pt.m_y = caxes.m_ymax - this.m_ystep * y;
    };
    
    this.viewport.prototype.get_index = function(x, y)
    {
        //alert("fractal::viewport::get_index()");
        return y * this.m_height + x;
    };
    
    
    
    // iteration result
    //___________________________________________________
    this.iter_result = function()
    {
        //alert("fractal::iter_result::ctor()");
        
        this.m_min_distance = 0;
    };
}


fractal.prototype.complex_add = function(pt0, pt1)
{
    pt0.m_x = pt0.m_x + pt1.m_x;
    pt0.m_y = pt0.m_y + pt1.m_y;
    return pt0;
};


fractal.prototype.complex_mul = function(pt0, pt1)
{
    var x = pt0.m_x * pt1.m_x - pt0.m_y * pt1.m_y;
    pt0.m_y = pt0.m_x * pt1.m_y + pt0.m_y * pt1.m_x;
    pt0.m_x = x;
    return pt0;
};


fractal.prototype.cantor_pack = function(n0, n1)
{
    var pack = Math.floor(0.5 * (n0 + n1) * (n0 + n1 + 1) + n1);
    return pack;
};

/*
 * unsigned long
cantor_packing(
    unsigned int n0,
    unsigned int n1
){
    unsigned long ret = 0.5 * (n0 + n1) * (n0 + n1 + 1) + n1;
 
    return ret;
}
 */


fractal.prototype.mbrot_iter = function(ir, z, c, imax)
{
    //alert("z.m_x = " + z.m_x + ", z.m_y = " + z.m_y);
    
    ir.m_min_distance = 99999999999999999;
    
    for (var i = 1; i < imax + 1; ++i)
    {
        this.complex_mul(z, z);
        this.complex_add(z, c);
        
        var d = z.distance();
        
        if (d != 0)
        {
            ir.m_min_distance = Math.min(ir.m_min_distance, d);
        }
        
        else
        {
            ir.m_min_distance = Math.min(0.1, d);
        }
        
        //alert("d = " + d);
        
        if (d > 2)
        {
            //alert("escape = " + d);
            return i;
        }
    }
    
    return imax;
};


fractal.prototype.mbrot_plane = function(vport, caxes, z, c, j, imax, callback)
{
    //alert("fractal::mbrot_plane()");
    
    vport.set_step(caxes);
    
    //alert("vport.m_width= " + vport.m_width);
    //alert("vport.m_height = " + vport.m_height);
    
    for (var y = 0; y < vport.m_height; y++)
    {
        for (var x = 0; x < vport.m_width; x++)
        {
            var i = vport.get_index(x, y); 
            vport.get_point(caxes, c, x, y);
            
            z.copy(c);
            
            if (! callback(z, j, i, imax)) return false;
        }
    }
    
    return true;
};

